@@ -429,6 +429,7 @@ struct sss_domain_info {
     char *dns_name;
     char *domain_id;
     uint32_t trust_direction;
+    uint32_t trust_type;
     struct timeval subdomains_last_checked;
 
     bool has_views;
@@ -776,7 +777,8 @@ int confdb_get_sub_sections(TALLOC_CTX *mem_ctx,
  * @return EINVAL - Typically internal processing error
  */
 int confdb_certmap_to_sysdb(struct confdb_ctx *cdb,
-                            struct sss_domain_info *dom);
+                            struct sss_domain_info *dom,
+                            bool certmaps_for_local_users);
 
 /**
  * @}
