@@ -186,8 +186,8 @@ class SSSDOptions(object):
         'dns_resolver_op_timeout': _('How long should keep trying to resolve single DNS query (seconds)'),
         'dns_resolver_timeout': _('How long to wait for replies from DNS when resolving servers (seconds)'),
         'dns_discovery_domain': _('The domain part of service discovery DNS query'),
-        'failover_primary_timeout': _('How often SSSD tries to reconnect to the primary server after a successful '
-                                      'connection to the backup server.'),
+        'failover_primary_timeout': _('Specifies the interval, in seconds, that SSSD waits before attempting to reconnect to the primary '
+                                      'server after a successful connection to the backup server'),
         'override_gid': _('Override GID value from the identity provider with this value'),
         'case_sensitive': _('Treat usernames as case sensitive'),
         'entry_cache_user_timeout': _('Entry cache timeout length (seconds)'),
@@ -200,6 +200,8 @@ class SSSDOptions(object):
         'refresh_expired_interval': _('How often should expired entries be refreshed in background'),
         'refresh_expired_interval_offset': _("Maximum period deviation when refreshing expired entries in background"),
         'dyndns_update': _("Whether to automatically update the client's DNS entry"),
+        'dyndns_update_per_family': _('Whether DNS update of A and AAAA record should be performed '
+                                      'in one update or in two separate updates'),
         'dyndns_ttl': _("The TTL to apply to the client's DNS entry after updating it"),
         'dyndns_iface': _("The interface whose IP should be used for dynamic DNS updates"),
         'dyndns_refresh_interval': _("How often to periodically update the client's DNS entry"),
@@ -208,6 +210,9 @@ class SSSDOptions(object):
         'dyndns_force_tcp': _("Whether the nsupdate utility should default to using TCP"),
         'dyndns_auth': _("What kind of authentication should be used to perform the DNS update"),
         'dyndns_server': _("Override the DNS server used to perform the DNS update"),
+        'dyndns_dot_cacert': _("The file of the certificate authorities certificates for DoT"),
+        'dyndns_dot_cert': _("The certificate(s) file for authentication for the DoT transport"),
+        'dyndns_dot_key': _("The key file for authenticated encryption for the DoT transport"),
         'subdomain_enumerate': _('Control enumeration of trusted domains'),
         'subdomain_refresh_interval': _('How often should subdomains list be refreshed'),
         'subdomain_refresh_interval_offset': _('Maximum period deviation when refreshing the subdomain list'),
@@ -357,6 +362,7 @@ class SSSDOptions(object):
         'ldap_uri': _('ldap_uri, The URI of the LDAP server'),
         'ldap_backup_uri': _('ldap_backup_uri, The URI of the LDAP server'),
         'ldap_search_base': _('The default base DN'),
+        'ldap_read_rootdse': _('How to read rootDSE from LDAP server'),
         'ldap_schema': _('The Schema Type in use on the LDAP server, rfc2307'),
         'ldap_pwmodify_mode': _('Mode used to change user password'),
         'ldap_default_bind_dn': _('The default bind DN'),
