@@ -581,6 +581,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'dns_discovery_domain',
             'failover_primary_timeout',
             'dyndns_update',
+            'dyndns_update_per_family',
             'dyndns_ttl',
             'dyndns_iface',
             'dyndns_refresh_interval',
@@ -589,6 +590,9 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'dyndns_force_tcp',
             'dyndns_auth',
             'dyndns_server',
+            'dyndns_dot_cacert',
+            'dyndns_dot_cert',
+            'dyndns_dot_key',
             'subdomain_enumerate',
             'override_gid',
             'case_sensitive',
@@ -942,6 +946,7 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'dns_discovery_domain',
             'failover_primary_timeout',
             'dyndns_update',
+            'dyndns_update_per_family',
             'dyndns_ttl',
             'dyndns_iface',
             'dyndns_refresh_interval',
@@ -949,6 +954,9 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'dyndns_force_tcp',
             'dyndns_auth',
             'dyndns_server',
+            'dyndns_dot_cacert',
+            'dyndns_dot_cert',
+            'dyndns_dot_key',
             'subdomain_enumerate',
             'override_gid',
             'case_sensitive',
