@@ -376,57 +376,6 @@ static errno_t sysdb_cache_create_empty(struct ldb_context *ldb,
     return EOK;
 }
 
-static errno_t sysdb_ts_cache_upgrade(TALLOC_CTX *mem_ctx,
-                                      struct sysdb_ctx *sysdb,
-                                      struct ldb_context *ldb,
-                                      struct sss_domain_info *domain,
-                                      const char *cur_version,
-                                      const char **_new_version)
-{
-    errno_t ret;
-    TALLOC_CTX *tmp_ctx;
-    const char *version;
-    struct ldb_context *save_ldb;
-
-    tmp_ctx = talloc_new(NULL);
-    if (tmp_ctx == NULL) {
-        return ENOMEM;
-    }
-
-    /* The upgrade process depends on having ldb around, yet the upgrade
-     * function shouldn't set the ldb pointer, only the connect function
-     * should after it's successful. To avoid hard refactoring, save the
-     * ldb pointer here and restore in the 'done' handler
-     */
-    save_ldb = sysdb->ldb;
-    sysdb->ldb = ldb;
-
-    version = talloc_strdup(tmp_ctx, cur_version);
-    if (version == NULL) {
-        ret = ENOMEM;
-        goto done;
-    }
-
-    DEBUG(SSSDBG_CONF_SETTINGS,
-          "Upgrading timstamp cache of DB [%s] from version: %s\n",
-          domain->name, version);
-
-    if (strcmp(version, SYSDB_TS_VERSION_0_1) == 0) {
-        ret = sysdb_ts_upgrade_01(sysdb, &version);
-        if (ret != EOK) {
-            goto done;
-        }
-    }
-
-    ret = EOK;
-
-done:
-    sysdb->ldb = save_ldb;
-    *_new_version = version;
-    talloc_free(tmp_ctx);
-    return ret;
-}
-
 static errno_t sysdb_domain_cache_upgrade(TALLOC_CTX *mem_ctx,
                                           struct sysdb_ctx *sysdb,
                                           struct sysdb_dom_upgrade_ctx *upgrade_ctx,
@@ -610,6 +559,13 @@ static errno_t sysdb_domain_cache_upgrade(TALLOC_CTX *mem_ctx,
         }
     }
 
+    if (strcmp(version, SYSDB_VERSION_0_24) == 0) {
+        ret = sysdb_upgrade_24(sysdb, &version);
+        if (ret != EOK) {
+            goto done;
+        }
+    }
+
     ret = EOK;
 done:
     sysdb->ldb = save_ldb;
@@ -816,6 +772,7 @@ static int sysdb_domain_cache_connect(struct sysdb_ctx *sysdb,
         ret = sysdb_domain_cache_upgrade(tmp_ctx, sysdb, upgrade_ctx,
                                          ldb, domain, version, &version);
         if (ret != EOK) {
+            DEBUG(SSSDBG_TRACE_FUNC, "sysdb_domain_cache_upgrade() failed\n");
             goto done;
         }
 
@@ -884,56 +841,6 @@ static int sysdb_timestamp_cache_connect(struct sysdb_ctx *sysdb,
     }
 
     ret = sysdb_ts_cache_connect(tmp_ctx, sysdb, domain, &ldb, &version);
-    switch (ret) {
-    case ERR_SYSDB_VERSION_TOO_OLD:
-        if (upgrade_ctx == NULL) {
-            DEBUG(SSSDBG_FATAL_FAILURE,
-                  "DB version too old [%s], expected [%s] for domain %s!\n",
-                   version, SYSDB_VERSION, domain->name);
-            break;
-        }
-
-        ret = sysdb_ts_cache_upgrade(tmp_ctx, sysdb, ldb, domain, version,
-                                     &version);
-        if (ret != EOK) {
-            DEBUG(SSSDBG_MINOR_FAILURE,
-                  "Could not upgrade the timestamp ldb file (%d) (%s)\n",
-                  ret, sss_strerror(ret));
-            break;
-        }
-
-        /* The version should now match SYSDB_VERSION.
-         * If not, it means we didn't match any of the
-         * known older versions. The DB might be
-         * corrupt or generated by a newer version of
-         * SSSD.
-         */
-        ret = sysdb_version_check(SYSDB_TS_VERSION, version);
-        if (ret == EOK) {
-            /* The cache has been upgraded.
-             * We need to reopen the LDB to ensure that
-             * any changes made above take effect.
-             */
-            ret = sysdb_ldb_reconnect(tmp_ctx,
-                                      sysdb->ldb_ts_file,
-                                      LDB_FLG_NOSYNC,
-                                      &ldb);
-            if (ret != EOK) {
-                DEBUG(SSSDBG_MINOR_FAILURE,
-                      "Could not reopen the timestamp ldb file (%d) (%s)\n",
-                      ret, sss_strerror(ret));
-            }
-        }
-        break;
-    case ERR_SYSDB_VERSION_TOO_NEW:
-        DEBUG(SSSDBG_MINOR_FAILURE,
-              "DB version too new [%s], expected [%s] for domain %s!\n",
-              version, SYSDB_TS_VERSION, domain->name);
-        break;
-    default:
-        break;
-    }
-
     if (ret != EOK) {
         DEBUG(SSSDBG_MINOR_FAILURE,
               "The timestamps cache could not be opened. "
@@ -953,7 +860,7 @@ static int sysdb_timestamp_cache_connect(struct sysdb_ctx *sysdb,
         ret = sysdb_ts_cache_connect(tmp_ctx, sysdb, domain, &ldb, &version);
         if (ret != EOK) {
             DEBUG(SSSDBG_MINOR_FAILURE,
-                  "Could not delete the timestamp ldb file (%d) (%s)\n",
+                  "sysdb_ts_cache_connect() failed after cache deletion [%d]: %s\n",
                   ret, sss_strerror(ret));
         }
     }
