@@ -814,6 +814,7 @@ static errno_t sysdb_enum_dn_filter(TALLOC_CTX *mem_ctx,
 {
     TALLOC_CTX *tmp_ctx = NULL;
     char *dn_filter;
+    char *sanitized_dn;
     const char *fqname;
     errno_t ret;
 
@@ -844,11 +845,18 @@ static errno_t sysdb_enum_dn_filter(TALLOC_CTX *mem_ctx,
     }
 
     for (size_t i = 0; i < ts_res->count; i++) {
+        ret = sss_filter_sanitize_dn(tmp_ctx,
+                                     ldb_dn_get_linearized(ts_res->msgs[i]->dn),
+                                     &sanitized_dn);
+        if (ret != EOK) {
+            goto done;
+        }
         dn_filter = talloc_asprintf_append(
                                   dn_filter,
                                   "(%s=%s)",
                                   SYSDB_DN,
-                                  ldb_dn_get_linearized(ts_res->msgs[i]->dn));
+                                  sanitized_dn);
+        talloc_free(sanitized_dn);
         if (dn_filter == NULL) {
             ret = ENOMEM;
             goto done;
@@ -1142,8 +1150,7 @@ int sysdb_getgrnam_with_views(TALLOC_CTX *mem_ctx,
 
         /* Must be called even without views to check to
          * SYSDB_DEFAULT_OVERRIDE_NAME */
-        ret = sysdb_add_group_member_overrides(domain, orig_obj->msgs[0],
-                                               DOM_HAS_VIEWS(domain));
+        ret = sysdb_add_group_member_overrides(domain, orig_obj->msgs[0]);
         if (ret != EOK) {
             DEBUG(SSSDBG_OP_FAILURE,
                   "sysdb_add_group_member_overrides failed.\n");
@@ -1176,7 +1183,7 @@ int sysdb_getgrnam(TALLOC_CTX *mem_ctx,
                    struct ldb_result **_res)
 {
     TALLOC_CTX *tmp_ctx;
-    static const char *attrs[] = SYSDB_GRSRC_ATTRS;
+    const char **attrs = SYSDB_GRSRC_ATTRS(domain);
     const char *fmt_filter;
     char *sanitized_name;
     struct ldb_dn *base_dn;
@@ -1225,7 +1232,9 @@ int sysdb_getgrnam(TALLOC_CTX *mem_ctx,
                     res->msgs[0], ORIGINALAD_PREFIX SYSDB_NAME, NULL);
 
             if (originalad_sanitized_name != NULL
-                    && strcmp(originalad_sanitized_name, sanitized_name) != 0) {
+                    && !sss_string_equal(domain->case_sensitive,
+                                         originalad_sanitized_name,
+                                         sanitized_name)) {
                 fmt_filter = SYSDB_GRNAM_FILTER;
                 base_dn = sysdb_group_base_dn(tmp_ctx, domain);
                 res = NULL;
@@ -1335,8 +1344,7 @@ int sysdb_getgrgid_with_views(TALLOC_CTX *mem_ctx,
 
         /* Must be called even without views to check to
          * SYSDB_DEFAULT_OVERRIDE_NAME */
-        ret = sysdb_add_group_member_overrides(domain, orig_obj->msgs[0],
-                                               DOM_HAS_VIEWS(domain));
+        ret = sysdb_add_group_member_overrides(domain, orig_obj->msgs[0]);
         if (ret != EOK) {
             DEBUG(SSSDBG_OP_FAILURE,
                   "sysdb_add_group_member_overrides failed.\n");
@@ -1376,7 +1384,7 @@ int sysdb_getgrgid_attrs(TALLOC_CTX *mem_ctx,
     struct ldb_dn *base_dn;
     struct ldb_result *res = NULL;
     int ret;
-    static const char *default_attrs[] = SYSDB_GRSRC_ATTRS;
+    const char **default_attrs = SYSDB_GRSRC_ATTRS(domain);
     const char **attrs = NULL;
 
     tmp_ctx = talloc_new(NULL);
@@ -1482,7 +1490,7 @@ int sysdb_enumgrent_filter(TALLOC_CTX *mem_ctx,
                            struct ldb_result **_res)
 {
     TALLOC_CTX *tmp_ctx;
-    static const char *attrs[] = SYSDB_GRSRC_ATTRS;
+    const char **attrs = SYSDB_GRSRC_ATTRS(domain);
     const char *filter = NULL;
     const char *ts_filter = NULL;
     const char *base_filter;
@@ -1629,8 +1637,7 @@ int sysdb_enumgrent_filter_with_views(TALLOC_CTX *mem_ctx,
             }
         }
 
-        ret = sysdb_add_group_member_overrides(domain, res->msgs[c],
-                                               DOM_HAS_VIEWS(domain));
+        ret = sysdb_add_group_member_overrides(domain, res->msgs[c]);
         if (ret != EOK) {
             DEBUG(SSSDBG_OP_FAILURE,
                   "sysdb_add_group_member_overrides failed.\n");
