@@ -23,11 +23,13 @@
 */
 
 #include <errno.h>
+#include <ldb.h>
 
 #include "util/util.h"
 #include "util/sss_nss.h"
 #include "util/strtonum.h"
 #include "db/sysdb.h"
+#include "db/sysdb_private.h"
 #include "providers/ldap/ldap_common.h"
 #include "providers/ldap/sdap_async.h"
 #include "providers/ldap/sdap_async_ad.h"
@@ -37,13 +39,13 @@
 #include "providers/ipa/ipa_subdomains.h"
 
 static struct tevent_req *
-ipa_srv_ad_acct_send(TALLOC_CTX *mem_ctx,
-                     struct tevent_context *ev,
-                     struct ipa_id_ctx *ipa_ctx,
-                     struct sysdb_attrs *override_attrs,
-                     struct dp_id_data *ar);
+ipa_srv_acct_send(TALLOC_CTX *mem_ctx,
+                  struct tevent_context *ev,
+                  struct ipa_id_ctx *ipa_ctx,
+                  struct sysdb_attrs *override_attrs,
+                  struct dp_id_data *ar);
 static errno_t
-ipa_srv_ad_acct_recv(struct tevent_req *req, int *dp_error_out);
+ipa_srv_acct_recv(struct tevent_req *req, int *dp_error_out);
 
 struct ipa_subdomain_account_state {
     struct tevent_context *ev;
@@ -169,13 +171,13 @@ static void ipa_subdomain_account_connected(struct tevent_req *subreq)
         goto fail;
     }
 
-    subreq = ipa_get_ad_override_send(state, state->ev, state->ctx,
+    subreq = ipa_get_trusted_override_send(state, state->ev, state->ctx,
                           state->ipa_ctx->ipa_options,
                           dp_opt_get_string(state->ipa_ctx->ipa_options->basic,
                                             IPA_KRB5_REALM),
                           state->ipa_ctx->view_name, state->ar);
     if (subreq == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE, "ipa_get_ad_override_send failed.\n");
+        DEBUG(SSSDBG_OP_FAILURE, "ipa_get_trusted_override_send failed.\n");
         ret = ENOMEM;
         goto fail;
     }
@@ -204,8 +206,8 @@ static void ipa_subdomain_account_got_override(struct tevent_req *subreq)
     const char *anchor = NULL;
     struct dp_id_data *ar;
 
-    ret = ipa_get_ad_override_recv(subreq, &dp_error, state,
-                                   &state->override_attrs);
+    ret = ipa_get_trusted_override_recv(subreq, &dp_error, state,
+                                        &state->override_attrs);
     talloc_zfree(subreq);
     if (ret != EOK) {
         ret = sdap_id_op_done(state->op, ret, &dp_error);
@@ -337,8 +339,8 @@ static errno_t ipa_subdomain_account_get_original_step(struct tevent_req *req,
     struct tevent_req *subreq;
 
     if (state->ipa_server_mode) {
-        subreq = ipa_srv_ad_acct_send(state, state->ev, state->ipa_ctx,
-                                      state->override_attrs, ar);
+        subreq = ipa_srv_acct_send(state, state->ev, state->ipa_ctx,
+                                   state->override_attrs, ar);
     } else {
         subreq = ipa_get_subdom_acct_send(state, state->ev, state->ipa_ctx,
                                           state->override_attrs, ar);
@@ -367,7 +369,7 @@ static void ipa_subdomain_account_done(struct tevent_req *subreq)
     struct sss_domain_info *object_dom;
 
     if (state->ipa_server_mode) {
-        ret = ipa_srv_ad_acct_recv(subreq, &dp_error);
+        ret = ipa_srv_acct_recv(subreq, &dp_error);
     } else {
         ret = ipa_get_subdom_acct_recv(subreq, &dp_error);
     }
@@ -783,8 +785,8 @@ ipa_ad_gc_conn_list(TALLOC_CTX *mem_ctx, struct ipa_id_ctx *ipa_ctx,
     return clist;
 }
 
-/* IPA lookup for server mode. Directly to AD. */
-struct ipa_get_ad_acct_state {
+/* IPA lookup for server mode. AD or IPA subdomain */
+struct ipa_get_acct_state {
     int dp_error;
     struct tevent_context *ev;
     struct ipa_id_ctx *ipa_ctx;
@@ -793,32 +795,37 @@ struct ipa_get_ad_acct_state {
     char *object_sid;
     struct sysdb_attrs *override_attrs;
     struct ldb_message *obj_msg;
+    enum ipa_trust_type type;
 };
 
-static void ipa_get_ad_acct_ad_part_done(struct tevent_req *subreq);
-static void ipa_get_ad_override_done(struct tevent_req *subreq);
-static errno_t ipa_get_ad_apply_override_step(struct tevent_req *req);
-static errno_t ipa_get_ad_ipa_membership_step(struct tevent_req *req);
+static void ipa_get_trusted_acct_part_done(struct tevent_req *subreq);
+static void ipa_get_trusted_override_done(struct tevent_req *subreq);
+static errno_t ipa_get_trusted_apply_override_step(struct tevent_req *req);
+static errno_t ipa_get_trusted_ipa_membership_step(struct tevent_req *req);
 static void ipa_id_get_groups_overrides_done(struct tevent_req *subreq);
-static void ipa_get_ad_acct_done(struct tevent_req *subreq);
+static void ipa_get_trusted_acct_done(struct tevent_req *subreq);
 
+static enum ipa_trust_type
+ipa_get_trust_type(struct ipa_id_ctx *ipa_ctx,
+                   struct sss_domain_info *dom);
 static struct tevent_req *
 ipa_get_ad_acct_send(TALLOC_CTX *mem_ctx,
                      struct tevent_context *ev,
                      struct ipa_id_ctx *ipa_ctx,
                      struct sysdb_attrs *override_attrs,
-                     struct dp_id_data *ar)
+                     struct dp_id_data *ar,
+                     enum ipa_trust_type type)
 {
     errno_t ret;
     struct tevent_req *req;
     struct tevent_req *subreq;
-    struct ipa_get_ad_acct_state *state;
+    struct ipa_get_acct_state *state;
     struct sdap_domain *sdom;
     struct sdap_id_conn_ctx **clist;
     struct sdap_id_ctx *sdap_id_ctx;
     struct ad_id_ctx *ad_id_ctx;
 
-    req = tevent_req_create(mem_ctx, &state, struct ipa_get_ad_acct_state);
+    req = tevent_req_create(mem_ctx, &state, struct ipa_get_acct_state);
     if (req == NULL) return NULL;
 
     state->dp_error = -1;
@@ -826,6 +833,7 @@ ipa_get_ad_acct_send(TALLOC_CTX *mem_ctx,
     state->ipa_ctx = ipa_ctx;
     state->ar = ar;
     state->obj_msg = NULL;
+    state->type = type;
     state->override_attrs = override_attrs;
 
     /* This can only be a subdomain request, verify subdomain */
@@ -879,7 +887,7 @@ ipa_get_ad_acct_send(TALLOC_CTX *mem_ctx,
         ret = ENOMEM;
         goto fail;
     }
-    tevent_req_set_callback(subreq, ipa_get_ad_acct_ad_part_done, req);
+    tevent_req_set_callback(subreq, ipa_get_trusted_acct_part_done, req);
     return req;
 
 fail:
@@ -893,15 +901,111 @@ static struct ad_id_ctx *
 ipa_get_ad_id_ctx(struct ipa_id_ctx *ipa_ctx,
                   struct sss_domain_info *dom)
 {
-    struct ipa_ad_server_ctx *iter;
+    struct ipa_subdom_server_ctx *iter;
 
     DLIST_FOR_EACH(iter, ipa_ctx->server_mode->trusts) {
         if (iter->dom == dom) break;
     }
 
-    return (iter) ? iter->ad_id_ctx : NULL;
+    return (iter) ? iter->id_ctx.ad_id_ctx : NULL;
 }
 
+static struct ipa_id_ctx *
+ipa_get_ipa_id_ctx(struct ipa_id_ctx *ipa_ctx,
+                   struct sss_domain_info *dom)
+{
+    struct ipa_subdom_server_ctx *iter;
+
+    DLIST_FOR_EACH(iter, ipa_ctx->server_mode->trusts) {
+        if (iter->dom == dom) break;
+    }
+
+    return (iter) ? iter->id_ctx.ipa_id_ctx : NULL;
+}
+
+static enum ipa_trust_type
+ipa_get_trust_type(struct ipa_id_ctx *ipa_ctx,
+                   struct sss_domain_info *dom)
+{
+    struct ipa_subdom_server_ctx *iter;
+
+    DLIST_FOR_EACH(iter, ipa_ctx->server_mode->trusts) {
+        if (iter->dom == dom) break;
+    }
+
+    return iter->type;
+}
+
+static struct ipa_id_ctx *ipa_get_ipa_id_ctx(struct ipa_id_ctx *ipa_ctx,
+                                             struct sss_domain_info *dom);
+
+static struct tevent_req *
+ipa_get_ipa_acct_send(TALLOC_CTX *mem_ctx,
+                      struct tevent_context *ev,
+                      struct ipa_id_ctx *ipa_ctx,
+                      struct sysdb_attrs *override_attrs,
+                      struct dp_id_data *ar,
+                      enum ipa_trust_type type)
+{
+    errno_t ret;
+    struct tevent_req *req;
+    struct tevent_req *subreq;
+    struct ipa_get_acct_state *state;
+    struct sdap_domain *sdom;
+    struct sdap_id_ctx *sdap_id_ctx;
+    struct ipa_id_ctx *ipa_id_ctx;
+
+    req = tevent_req_create(mem_ctx, &state, struct ipa_get_acct_state);
+    if (req == NULL) return NULL;
+
+    state->dp_error = -1;
+    state->ev = ev;
+    state->ipa_ctx = ipa_ctx;
+    state->ar = ar;
+    state->obj_msg = NULL;
+    state->type = type;
+    state->override_attrs = override_attrs;
+
+    /* This can only be a subdomain request, verify subdomain */
+    state->obj_dom = find_domain_by_name(ipa_ctx->sdap_id_ctx->be->domain,
+                                         ar->domain, true);
+    if (state->obj_dom == NULL) {
+        ret = EINVAL;
+        goto fail;
+    }
+
+    /* Let's see if this subdomain has a ipa_id_ctx */
+    ipa_id_ctx = ipa_get_ipa_id_ctx(ipa_ctx, state->obj_dom);
+    if (ipa_id_ctx == NULL) {
+        ret = EINVAL;
+        goto fail;
+    }
+    sdap_id_ctx = ipa_id_ctx->sdap_id_ctx;
+
+    /* Now we already need ipa_id_ctx in particular sdap_id_conn_ctx */
+    sdom = sdap_domain_get(sdap_id_ctx->opts, state->obj_dom);
+    if (sdom == NULL) {
+        ret = EIO;
+        goto fail;
+    }
+
+    ipa_id_ctx->sdap_id_ctx->opts->sdom = sdom;
+    subreq = ipa_id_get_account_info_send(req, ev, ipa_id_ctx, ar);
+    if (subreq == NULL) {
+        ret = ENOMEM;
+        goto fail;
+    }
+    tevent_req_set_callback(subreq, ipa_get_trusted_acct_part_done, req);
+    return req;
+
+fail:
+    state->dp_error = DP_ERR_FATAL;
+    tevent_req_error(req, ret);
+    tevent_req_post(req, ev);
+    return req;
+}
+
+
 static errno_t
 get_subdomain_homedir_of_user(TALLOC_CTX *mem_ctx, struct sss_domain_info *dom,
                               const char *fqname, uint32_t uid,
@@ -1257,18 +1361,123 @@ done:
     return ret;
 }
 
+static void ipa_get_sid_ipa_next(struct tevent_req *subreq)
+{
+    int ret;
+    int dp_error = DP_ERR_FATAL;
+    const char *sid;
+    const char *user;
+    struct ldb_message *user_msg;
+    struct dp_id_data *ar;
+    struct dp_id_data *user_ar;
+    struct tevent_req *req = tevent_req_callback_data(subreq,
+                                                      struct tevent_req);
+    struct ipa_get_acct_state *state = tevent_req_data(req,
+                                                struct ipa_get_acct_state);
+
+    ret = ipa_subdomain_account_recv(subreq, &state->dp_error);
+    talloc_zfree(subreq);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "ipa_id_get_account_info failed: %d %d\n", ret,
+                                 dp_error);
+        goto done;
+    }
+
+    /* Get username object and id data */
+    user = ldb_msg_find_attr_as_string(state->obj_msg, SYSDB_NAME, NULL);
+    if (user == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Cannot find SYSDB_NAME.\n");
+        ret = ENOMEM;
+        goto done;
+    }
+
+    DEBUG(SSSDBG_TRACE_INTERNAL,
+          "Looking up IPA object [%s] from LDAP.\n",
+          user);
+    ret = get_dp_id_data_for_user_name(state,
+                                       user,
+                                       state->obj_dom->name,
+                                       &user_ar);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE,
+              "Failed to create lookup data for IPA object [%s].\n",
+              user);
+        goto done;
+    }
+
+    /* Get user object */
+    ret = get_object_from_cache(state, state->obj_dom, user_ar,
+                                &user_msg);
+    if (ret == ENOENT) {
+        DEBUG(SSSDBG_MINOR_FAILURE, "Object not found, ending request\n");
+        tevent_req_done(req);
+        return;
+    } else if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "get_object_from_cache failed.\n");
+        goto done;
+    }
+
+    /* Retrieve SID from user object */
+    sid = ldb_msg_find_attr_as_string(user_msg, SYSDB_SID_STR, NULL);
+    if (sid == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Cannot find a SID.\n");
+    }
+
+    state->object_sid = talloc_strdup(state, sid);
+    if (state->object_sid == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "talloc_strdup failed.\n");
+        ret = ENOMEM;
+        goto done;
+    }
+
+    ret = get_dp_id_data_for_sid(state, state->object_sid,
+                                 state->obj_dom->name, &ar);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "get_dp_id_data_for_sid failed.\n");
+        goto done;
+    }
+
+    subreq = ipa_get_trusted_override_send(state, state->ev,
+                                           state->ipa_ctx->sdap_id_ctx,
+                                           state->ipa_ctx->ipa_options,
+                                           state->ipa_ctx->server_mode->realm,
+                                           state->ipa_ctx->view_name,
+                                           ar);
+    if (subreq == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "ipa_get_ad_override_send failed.\n");
+        ret = ENOMEM;
+        goto done;
+    }
+    tevent_req_set_callback(subreq, ipa_get_trusted_override_done, req);
+
+    return;
+
+done:
+    tevent_req_error(req,ret);
+    return;
+}
+
 static void
-ipa_get_ad_acct_ad_part_done(struct tevent_req *subreq)
+ipa_get_trusted_acct_part_done(struct tevent_req *subreq)
 {
     struct tevent_req *req = tevent_req_callback_data(subreq,
                                                 struct tevent_req);
-    struct ipa_get_ad_acct_state *state = tevent_req_data(req,
-                                                struct ipa_get_ad_acct_state);
+    struct ipa_get_acct_state *state = tevent_req_data(req,
+                                                struct ipa_get_acct_state);
     errno_t ret;
     const char *sid;
+    const char *user;
     struct dp_id_data *ar;
+    struct dp_id_data *user_ar;
+
+    if (state->type == IPA_TRUST_AD) {
+        ret = ad_handle_acct_info_recv(subreq, &state->dp_error, NULL);
+    } else if (state->type == IPA_TRUST_IPA) {
+        ret = ipa_id_get_account_info_recv(subreq, &state->dp_error);
+    } else {
+        ret = EINVAL;
+    }
 
-    ret = ad_handle_acct_info_recv(subreq, &state->dp_error, NULL);
     talloc_zfree(subreq);
     if (ret == ERR_SUBDOM_INACTIVE) {
         tevent_req_error(req, ret);
@@ -1302,9 +1511,45 @@ ipa_get_ad_acct_ad_part_done(struct tevent_req *subreq)
     if (state->override_attrs == NULL) {
         sid = ldb_msg_find_attr_as_string(state->obj_msg, SYSDB_SID_STR, NULL);
         if (sid == NULL) {
-            DEBUG(SSSDBG_CRIT_FAILURE, "Cannot find a SID.\n");
-            ret = EINVAL;
-            goto fail;
+            DEBUG(SSSDBG_MINOR_FAILURE, "Cannot find a SID.\n");
+            /* In IPA-IPA trust, user private groups do not contain a SID. Lookup the
+             * equivalent user object of the same name in IPA and use this SID instead */
+            if (state->type == IPA_TRUST_IPA) {
+                user = ldb_msg_find_attr_as_string(state->obj_msg, SYSDB_NAME, NULL);
+                if (user == NULL) {
+                    DEBUG(SSSDBG_CRIT_FAILURE, "Cannot find SYSDB_NAME.\n");
+                    ret = ENOMEM;
+                    goto fail;
+                }
+
+                DEBUG(SSSDBG_TRACE_INTERNAL,
+                      "Looking up IPA object [%s] from LDAP.\n",
+                      user);
+                ret = get_dp_id_data_for_user_name(state,
+                                                   user,
+                                                   state->obj_dom->name,
+                                                   &user_ar);
+                if (ret != EOK) {
+                    DEBUG(SSSDBG_OP_FAILURE,
+                          "Failed to create lookup data for IPA object [%s].\n",
+                          user);
+                    goto fail;
+                }
+
+                subreq = ipa_subdomain_account_send(state, state->ev, state->ipa_ctx, user_ar);
+                if (subreq == NULL) {
+                    DEBUG(SSSDBG_OP_FAILURE,
+                          "ipa_id_get_account_info_send failed.\n");
+                    ret = ENOMEM;
+                    goto fail;
+                }
+                tevent_req_set_callback(subreq, ipa_get_sid_ipa_next, req);
+            } else {
+                ret = EINVAL;
+                goto fail;
+            }
+
+            return;
         }
 
         state->object_sid = talloc_strdup(state, sid);
@@ -1321,23 +1566,23 @@ ipa_get_ad_acct_ad_part_done(struct tevent_req *subreq)
             goto fail;
         }
 
-        subreq = ipa_get_ad_override_send(state, state->ev,
-                                          state->ipa_ctx->sdap_id_ctx,
-                                          state->ipa_ctx->ipa_options,
-                                          state->ipa_ctx->server_mode->realm,
-                                          state->ipa_ctx->view_name,
-                                          ar);
+        subreq = ipa_get_trusted_override_send(state, state->ev,
+                                               state->ipa_ctx->sdap_id_ctx,
+                                               state->ipa_ctx->ipa_options,
+                                               state->ipa_ctx->server_mode->realm,
+                                               state->ipa_ctx->view_name,
+                                               ar);
         if (subreq == NULL) {
-            DEBUG(SSSDBG_OP_FAILURE, "ipa_get_ad_override_send failed.\n");
+            DEBUG(SSSDBG_OP_FAILURE, "ipa_get_trusted_override_send failed.\n");
             ret = ENOMEM;
             goto fail;
         }
-        tevent_req_set_callback(subreq, ipa_get_ad_override_done, req);
+        tevent_req_set_callback(subreq, ipa_get_trusted_override_done, req);
     } else {
-        ret = ipa_get_ad_apply_override_step(req);
+        ret = ipa_get_trusted_apply_override_step(req);
         if (ret != EOK) {
             DEBUG(SSSDBG_OP_FAILURE,
-                  "ipa_get_ad_apply_override_step failed.\n");
+                  "ipa_get_trusted_apply_override_step failed.\n");
             goto fail;
         }
     }
@@ -1352,15 +1597,15 @@ fail:
 
 
 static void
-ipa_get_ad_override_done(struct tevent_req *subreq)
+ipa_get_trusted_override_done(struct tevent_req *subreq)
 {
     struct tevent_req *req = tevent_req_callback_data(subreq,
                                                 struct tevent_req);
-    struct ipa_get_ad_acct_state *state = tevent_req_data(req,
-                                                struct ipa_get_ad_acct_state);
+    struct ipa_get_acct_state *state = tevent_req_data(req,
+                                                struct ipa_get_acct_state);
     errno_t ret;
 
-    ret = ipa_get_ad_override_recv(subreq, &state->dp_error, state,
+    ret = ipa_get_trusted_override_recv(subreq, &state->dp_error, state,
                                    &state->override_attrs);
     talloc_zfree(subreq);
     if (ret != EOK) {
@@ -1370,9 +1615,9 @@ ipa_get_ad_override_done(struct tevent_req *subreq)
 
     }
 
-    ret = ipa_get_ad_apply_override_step(req);
+    ret = ipa_get_trusted_apply_override_step(req);
     if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, "ipa_get_ad_apply_override_step failed.\n");
+        DEBUG(SSSDBG_OP_FAILURE, "ipa_get_trusted_apply_override_step failed.\n");
         goto fail;
     }
 
@@ -1387,8 +1632,8 @@ fail:
 static void ipa_check_ghost_members_done(struct tevent_req *subreq);
 static errno_t ipa_check_ghost_members(struct tevent_req *req)
 {
-    struct ipa_get_ad_acct_state *state = tevent_req_data(req,
-                                                struct ipa_get_ad_acct_state);
+    struct ipa_get_acct_state *state = tevent_req_data(req,
+                                                struct ipa_get_acct_state);
     errno_t ret;
     struct tevent_req *subreq;
     struct ldb_message_element *ghosts = NULL;
@@ -1445,10 +1690,10 @@ static void ipa_check_ghost_members_done(struct tevent_req *subreq)
     return;
 }
 
-static errno_t ipa_get_ad_apply_override_step(struct tevent_req *req)
+static errno_t ipa_get_trusted_apply_override_step(struct tevent_req *req)
 {
-    struct ipa_get_ad_acct_state *state = tevent_req_data(req,
-                                                struct ipa_get_ad_acct_state);
+    struct ipa_get_acct_state *state = tevent_req_data(req,
+                                                struct ipa_get_acct_state);
     errno_t ret;
     struct tevent_req *subreq;
     const char *obj_name;
@@ -1554,9 +1799,9 @@ static errno_t ipa_get_ad_apply_override_step(struct tevent_req *req)
         return EOK;
     }
 
-    ret = ipa_get_ad_ipa_membership_step(req);
+    ret = ipa_get_trusted_ipa_membership_step(req);
     if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, "ipa_get_ad_ipa_membership_step failed.\n");
+        DEBUG(SSSDBG_OP_FAILURE, "ipa_get_trusted_ipa_membership_step failed.\n");
         return ret;
     }
 
@@ -1578,9 +1823,9 @@ static void ipa_id_get_groups_overrides_done(struct tevent_req *subreq)
         return;
     }
 
-    ret = ipa_get_ad_ipa_membership_step(req);
+    ret = ipa_get_trusted_ipa_membership_step(req);
     if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, "ipa_get_ad_ipa_membership_step failed.\n");
+        DEBUG(SSSDBG_OP_FAILURE, "ipa_get_trusted_ipa_membership_step failed.\n");
         tevent_req_error(req, ret);
         return;
     }
@@ -1588,39 +1833,39 @@ static void ipa_id_get_groups_overrides_done(struct tevent_req *subreq)
     return;
 }
 
-static errno_t ipa_get_ad_ipa_membership_step(struct tevent_req *req)
+static errno_t ipa_get_trusted_ipa_membership_step(struct tevent_req *req)
 {
-    struct ipa_get_ad_acct_state *state = tevent_req_data(req,
-                                                struct ipa_get_ad_acct_state);
+    struct ipa_get_acct_state *state = tevent_req_data(req,
+                                                struct ipa_get_acct_state);
     struct tevent_req *subreq;
 
-    /* For initgroups request we have to check IPA group memberships of AD
+    /* For initgroups request we have to check IPA group memberships of trusted
      * users. This has to be done for other user-request as well to make sure
      * IPA related attributes are not overwritten. */
-    subreq = ipa_get_ad_memberships_send(state, state->ev, state->ar,
-                                         state->ipa_ctx->server_mode,
-                                         state->obj_dom,
-                                         state->ipa_ctx->sdap_id_ctx,
-                                         state->ipa_ctx->server_mode->realm);
+    subreq = ipa_get_trusted_memberships_send(state, state->ev, state->ar,
+                                              state->ipa_ctx->server_mode,
+                                              state->obj_dom,
+                                              state->ipa_ctx->sdap_id_ctx,
+                                              state->ipa_ctx->server_mode->realm);
     if (subreq == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE, "ipa_get_ad_memberships_send failed.\n");
+        DEBUG(SSSDBG_OP_FAILURE, "ipa_get_trusted_memberships_send failed.\n");
         return ENOMEM;
     }
-    tevent_req_set_callback(subreq, ipa_get_ad_acct_done, req);
+    tevent_req_set_callback(subreq, ipa_get_trusted_acct_done, req);
 
     return EOK;
 }
 
 static void
-ipa_get_ad_acct_done(struct tevent_req *subreq)
+ipa_get_trusted_acct_done(struct tevent_req *subreq)
 {
     struct tevent_req *req = tevent_req_callback_data(subreq,
                                                 struct tevent_req);
-    struct ipa_get_ad_acct_state *state = tevent_req_data(req,
-                                                struct ipa_get_ad_acct_state);
+    struct ipa_get_acct_state *state = tevent_req_data(req,
+                                                struct ipa_get_acct_state);
     errno_t ret;
 
-    ret = ipa_get_ad_memberships_recv(subreq, &state->dp_error);
+    ret = ipa_get_trusted_memberships_recv(subreq, &state->dp_error);
     talloc_zfree(subreq);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, "IPA external groups lookup failed: %d\n",
@@ -1634,10 +1879,10 @@ ipa_get_ad_acct_done(struct tevent_req *subreq)
 }
 
 static errno_t
-ipa_get_ad_acct_recv(struct tevent_req *req, int *dp_error_out)
+ipa_get_acct_recv(struct tevent_req *req, int *dp_error_out)
 {
-    struct ipa_get_ad_acct_state *state = tevent_req_data(req,
-                                                struct ipa_get_ad_acct_state);
+    struct ipa_get_acct_state *state = tevent_req_data(req,
+                                                struct ipa_get_acct_state);
 
     if (dp_error_out) {
         *dp_error_out = state->dp_error;
@@ -1648,7 +1893,7 @@ ipa_get_ad_acct_recv(struct tevent_req *req, int *dp_error_out)
     return EOK;
 }
 
-struct ipa_srv_ad_acct_state {
+struct ipa_srv_acct_state {
     struct tevent_context *ev;
     struct ipa_id_ctx *ipa_ctx;
     struct sysdb_attrs *override_attrs;
@@ -1656,27 +1901,28 @@ struct ipa_srv_ad_acct_state {
 
     struct sss_domain_info *obj_dom;
     struct be_ctx *be_ctx;
+    enum ipa_trust_type type;
     bool retry;
 
     int dp_error;
 };
 
-static int ipa_srv_ad_acct_lookup_step(struct tevent_req *req);
-static void ipa_srv_ad_acct_lookup_done(struct tevent_req *subreq);
-static void ipa_srv_ad_acct_retried(struct tevent_req *subreq);
+static int ipa_srv_acct_lookup_step(struct tevent_req *req);
+static void ipa_srv_acct_lookup_done(struct tevent_req *subreq);
+static void ipa_srv_acct_retried(struct tevent_req *subreq);
 
 static struct tevent_req *
-ipa_srv_ad_acct_send(TALLOC_CTX *mem_ctx,
-                     struct tevent_context *ev,
-                     struct ipa_id_ctx *ipa_ctx,
-                     struct sysdb_attrs *override_attrs,
-                     struct dp_id_data *ar)
+ipa_srv_acct_send(TALLOC_CTX *mem_ctx,
+                  struct tevent_context *ev,
+                  struct ipa_id_ctx *ipa_ctx,
+                  struct sysdb_attrs *override_attrs,
+                  struct dp_id_data *ar)
 {
     errno_t ret;
     struct tevent_req *req;
-    struct ipa_srv_ad_acct_state *state;
+    struct ipa_srv_acct_state *state;
 
-    req = tevent_req_create(mem_ctx, &state, struct ipa_srv_ad_acct_state);
+    req = tevent_req_create(mem_ctx, &state, struct ipa_srv_acct_state);
     if (req == NULL) {
         return NULL;
     }
@@ -1698,7 +1944,9 @@ ipa_srv_ad_acct_send(TALLOC_CTX *mem_ctx,
         goto fail;
     }
 
-    ret = ipa_srv_ad_acct_lookup_step(req);
+    state->type = ipa_get_trust_type(state->ipa_ctx, state->obj_dom);
+
+    ret = ipa_srv_acct_lookup_step(req);
     if (ret != EOK) {
         goto fail;
     }
@@ -1711,34 +1959,45 @@ fail:
     return req;
 }
 
-static int ipa_srv_ad_acct_lookup_step(struct tevent_req *req)
+static int ipa_srv_acct_lookup_step(struct tevent_req *req)
 {
     struct tevent_req *subreq;
-    struct ipa_srv_ad_acct_state *state = tevent_req_data(req,
-                                            struct ipa_srv_ad_acct_state);
+    struct ipa_srv_acct_state *state = tevent_req_data(req,
+                                            struct ipa_srv_acct_state);
+
+    if (state->type == IPA_TRUST_AD) {
+        DEBUG(SSSDBG_TRACE_FUNC, "Looking up AD account\n");
+        subreq = ipa_get_ad_acct_send(state, state->ev, state->ipa_ctx,
+                                      state->override_attrs,
+                                      state->ar, state->type);
+    } else if (state->type == IPA_TRUST_IPA) {
+        DEBUG(SSSDBG_TRACE_FUNC, "Looking up IPA account\n");
+        subreq = ipa_get_ipa_acct_send(state, state->ev, state->ipa_ctx,
+                                       state->override_attrs,
+                                       state->ar, state->type);
+    } else {
+        DEBUG(SSSDBG_TRACE_FUNC, "Trust type is unknown\n");
+        subreq = NULL;
+    }
 
-    DEBUG(SSSDBG_TRACE_FUNC, "Looking up AD account\n");
-    subreq = ipa_get_ad_acct_send(state, state->ev, state->ipa_ctx,
-                                  state->override_attrs,
-                                  state->ar);
     if (subreq == NULL) {
         return ENOMEM;
     }
-    tevent_req_set_callback(subreq, ipa_srv_ad_acct_lookup_done, req);
+    tevent_req_set_callback(subreq, ipa_srv_acct_lookup_done, req);
 
     return EOK;
 }
 
-static void ipa_srv_ad_acct_lookup_done(struct tevent_req *subreq)
+static void ipa_srv_acct_lookup_done(struct tevent_req *subreq)
 {
     errno_t ret;
     int dp_error = DP_ERR_FATAL;
     struct tevent_req *req = tevent_req_callback_data(subreq,
                                                 struct tevent_req);
-    struct ipa_srv_ad_acct_state *state = tevent_req_data(req,
-                                            struct ipa_srv_ad_acct_state);
+    struct ipa_srv_acct_state *state = tevent_req_data(req,
+                                            struct ipa_srv_acct_state);
 
-    ret = ipa_get_ad_acct_recv(subreq, &dp_error);
+    ret = ipa_get_acct_recv(subreq, &dp_error);
     talloc_free(subreq);
     if (ret == ERR_SUBDOM_INACTIVE && state->retry == true) {
 
@@ -1753,7 +2012,7 @@ static void ipa_srv_ad_acct_lookup_done(struct tevent_req *subreq)
         if (subreq == NULL) {
             goto fail;
         }
-        tevent_req_set_callback(subreq, ipa_srv_ad_acct_retried, req);
+        tevent_req_set_callback(subreq, ipa_srv_acct_retried, req);
         return;
     } else if (ret != EOK) {
         be_mark_dom_offline(state->obj_dom, state->be_ctx);
@@ -1772,14 +2031,14 @@ fail:
     tevent_req_error(req, ret);
 }
 
-static void ipa_srv_ad_acct_retried(struct tevent_req *subreq)
+static void ipa_srv_acct_retried(struct tevent_req *subreq)
 {
     errno_t ret;
     struct ad_id_ctx *ad_id_ctx;
     struct tevent_req *req = tevent_req_callback_data(subreq,
                                                 struct tevent_req);
-    struct ipa_srv_ad_acct_state *state = tevent_req_data(req,
-                                            struct ipa_srv_ad_acct_state);
+    struct ipa_srv_acct_state *state = tevent_req_data(req,
+                                            struct ipa_srv_acct_state);
 
     ret = ipa_server_trusted_dom_setup_recv(subreq);
     talloc_free(subreq);
@@ -1802,7 +2061,7 @@ static void ipa_srv_ad_acct_retried(struct tevent_req *subreq)
 
     ad_failover_reset(state->be_ctx, ad_id_ctx->ad_options->service);
 
-    ret = ipa_srv_ad_acct_lookup_step(req);
+    ret = ipa_srv_acct_lookup_step(req);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE,
               "Failed to look up AD acct [%d]: %s\n", ret, sss_strerror(ret));
@@ -1813,10 +2072,10 @@ static void ipa_srv_ad_acct_retried(struct tevent_req *subreq)
 }
 
 static errno_t
-ipa_srv_ad_acct_recv(struct tevent_req *req, int *dp_error_out)
+ipa_srv_acct_recv(struct tevent_req *req, int *dp_error_out)
 {
-    struct ipa_srv_ad_acct_state *state = tevent_req_data(req,
-                                                struct ipa_srv_ad_acct_state);
+    struct ipa_srv_acct_state *state = tevent_req_data(req,
+                                                struct ipa_srv_acct_state);
 
     if (dp_error_out) {
         *dp_error_out = state->dp_error;
