@@ -27,6 +27,7 @@
 #include "providers/ipa/ipa_subdomains.h"
 #include "providers/ipa/ipa_common.h"
 #include "providers/ipa/ipa_id.h"
+#include "providers/ipa/ipa_srv.h"
 
 /* These constants are defined in MS-ADTS 6.1.6.7.1
  *  https://msdn.microsoft.com/en-us/library/cc223768.aspx
@@ -35,6 +36,10 @@
 #define LSA_TRUST_DIRECTION_OUTBOUND 0x00000002
 #define LSA_TRUST_DIRECTION_MASK (LSA_TRUST_DIRECTION_INBOUND | LSA_TRUST_DIRECTION_OUTBOUND)
 
+/* See ipatrust_encode_type() method in freeipa ipaserver/dcerpc_common.py */
+#define TRUST_TYPE_AD (1 << 4)
+#define TRUST_TYPE_IPA (1 << 5)
+
 static char *forest_keytab(TALLOC_CTX *mem_ctx, const char *forest)
 {
     return talloc_asprintf(mem_ctx,
@@ -55,6 +60,34 @@ static char *subdomain_trust_princ(TALLOC_CTX *mem_ctx,
                            sd->parent->flat_name, forest_realm);
 }
 
+struct sdap_domain *
+sdap_domain_get_by_trust_type(struct ipa_subdom_server_ctx *trust,
+                              struct sss_domain_info *dom)
+{
+    struct sdap_domain *sdom = NULL;
+
+    if (trust->type == IPA_TRUST_AD) {
+        sdom = sdap_domain_get(trust->id_ctx.ad_id_ctx->sdap_id_ctx->opts,
+                               dom);
+    } else if (trust->type == IPA_TRUST_AD) {
+        sdom = sdap_domain_get(trust->id_ctx.ipa_id_ctx->sdap_id_ctx->opts,
+                               dom);
+    }
+
+    return sdom;
+}
+
+void
+sdap_domain_remove_by_trust_type(struct ipa_subdom_server_ctx *trust,
+                                 struct sss_domain_info *subdom)
+{
+    if (trust->type == IPA_TRUST_AD) {
+        sdap_domain_remove(trust->id_ctx.ad_id_ctx->sdap_id_ctx->opts, subdom);
+    } else if (trust->type == IPA_TRUST_IPA) {
+        sdap_domain_remove(trust->id_ctx.ipa_id_ctx->sdap_id_ctx->opts, subdom);
+    }
+}
+
 static uint32_t default_direction(TALLOC_CTX *mem_ctx,
                                   struct ldb_context *ldb_ctx,
                                   struct sysdb_attrs *attrs)
@@ -108,6 +141,59 @@ errno_t ipa_server_get_trust_direction(struct sysdb_attrs *sd,
     return EOK;
 }
 
+/* See ipatrust_encode_type() method in freeipa ipaserver/dcerpc_common.py
+ *
+ *      ipaTrustType attribute will encode:
+ *      bits 0..1 -- direction of trust
+ *         bit 0  -- one-way incoming trust
+ *         bit 1  -- one-way outgoing trust
+ *      bits 4..7 -- type of trust
+ *         bit 4  -- trust to Active Directory
+ *         bit 5  -- trust to IPA
+ */
+enum ipa_trust_type ipa_server_decode_trust_type(uint32_t type)
+{
+    enum ipa_trust_type trust_type;
+
+    if (type & TRUST_TYPE_AD) {
+        trust_type = IPA_TRUST_AD;
+    } else if (type & TRUST_TYPE_IPA) {
+        trust_type = IPA_TRUST_IPA;
+    /* Always assume existing trusts are AD */
+    } else {
+        trust_type = IPA_TRUST_AD;
+    }
+
+    return trust_type;
+}
+
+errno_t ipa_server_get_trust_type(struct sysdb_attrs *sd,
+                                  struct ldb_context *ldb_ctx,
+                                  uint32_t *_type)
+{
+    uint32_t ipa_trust_type = 0;
+    uint32_t type;
+    int ret;
+
+    ret = sysdb_attrs_get_uint32_t(sd, IPA_PARTNER_TRUST_TYPE,
+                                   &ipa_trust_type);
+    DEBUG(SSSDBG_TRACE_INTERNAL,
+          "Raw %s value: %d\n", IPA_PARTNER_TRUST_TYPE, ipa_trust_type);
+    if (ret == ENOENT) {
+        /* Always assume existing trusts (without IPA_PARTNER_TRUST_TYPE) are AD */
+        type = IPA_TRUST_AD;
+    } else if (ret == EOK) {
+        /* Just store the value in SYSDB, we will check it while we're
+         * trying to use the trust */
+        type = ipa_server_decode_trust_type(ipa_trust_type);
+    } else {
+        return ret;
+    }
+
+    *_type = type;
+    return EOK;
+}
+
 const char *ipa_trust_dir2str(uint32_t direction)
 {
     if ((direction & LSA_TRUST_DIRECTION_OUTBOUND)
@@ -124,17 +210,65 @@ const char *ipa_trust_dir2str(uint32_t direction)
     return "unknown";
 }
 
+const char *ipa_trust_type2str(uint32_t type)
+{
+    if (type == IPA_TRUST_AD) {
+        return "Active Directory Trust";
+    } else if (type == IPA_TRUST_IPA) {
+        return "IPA Trust";
+    } else if (type == 0) {
+        return "not set";
+    }
+
+    return "unknown";
+}
+
 #ifndef IPA_GETKEYTAB_TIMEOUT
 #define IPA_GETKEYTAB_TIMEOUT 5
 #endif /* IPA_GETKEYTAB_TIMEOUT */
 
+static struct ipa_options *
+ipa_create_ipa_trust_ctx(struct ipa_id_ctx *id_ctx,
+                         struct be_ctx *be_ctx,
+                         const char *subdom_conf_path,
+                         const char *forest,
+                         const char *forest_realm,
+                         struct sss_domain_info *subdom)
+{
+    char *keytab;
+    char *principal;
+    struct ipa_options *ipa_options;
+
+    keytab = forest_keytab(id_ctx, forest);
+    principal = subdomain_trust_princ(id_ctx, forest_realm, subdom);
+    if (keytab == NULL || principal == NULL) {
+        return NULL;
+    }
+
+    ipa_options = ipa_create_trust_options(id_ctx,
+                                           be_ctx,
+                                           be_ctx->cdb,
+                                           subdom_conf_path,
+                                           be_ctx->provider,
+                                           subdom,
+                                           keytab,
+                                           principal);
+    if (ipa_options == NULL) {
+        talloc_free(keytab);
+        talloc_free(principal);
+        return NULL;
+    }
+
+    return ipa_options;
+}
+
 static struct ad_options *
-ipa_create_1way_trust_ctx(struct ipa_id_ctx *id_ctx,
-                          struct be_ctx *be_ctx,
-                          const char *subdom_conf_path,
-                          const char *forest,
-                          const char *forest_realm,
-                          struct sss_domain_info *subdom)
+ipa_create_ad_trust_ctx(struct ipa_id_ctx *id_ctx,
+                        struct be_ctx *be_ctx,
+                        const char *subdom_conf_path,
+                        const char *forest,
+                        const char *forest_realm,
+                        struct sss_domain_info *subdom)
 {
     char *keytab;
     char *principal;
@@ -146,14 +280,15 @@ ipa_create_1way_trust_ctx(struct ipa_id_ctx *id_ctx,
         return NULL;
     }
 
-    ad_options = ad_create_1way_trust_options(id_ctx,
-                                              be_ctx->cdb,
-                                              subdom_conf_path,
-                                              be_ctx->provider,
-                                              subdom,
-                                              id_ctx->server_mode->hostname,
-                                              keytab,
-                                              principal);
+    ad_options = ad_create_trust_options(id_ctx,
+                                         be_ctx->cdb,
+                                         subdom_conf_path,
+                                         be_ctx->provider,
+                                         subdom,
+                                         NULL,
+                                         id_ctx->server_mode->hostname,
+                                         keytab,
+                                         principal);
     if (ad_options == NULL) {
         talloc_free(keytab);
         talloc_free(principal);
@@ -190,9 +325,9 @@ static struct ad_options *ipa_ad_options_new(struct be_ctx *be_ctx,
      * thus we always should be initializing principals/keytabs
      * as if we are running one-way trust */
     if (direction & LSA_TRUST_DIRECTION_MASK) {
-        ad_options = ipa_create_1way_trust_ctx(id_ctx, be_ctx,
-                                               subdom_conf_path, forest,
-                                               forest_realm, subdom);
+        ad_options = ipa_create_ad_trust_ctx(id_ctx, be_ctx,
+                                             subdom_conf_path, forest,
+                                             forest_realm, subdom);
     } else {
         DEBUG(SSSDBG_CRIT_FAILURE, "Unsupported trust direction!\n");
         ad_options = NULL;
@@ -204,9 +339,9 @@ static struct ad_options *ipa_ad_options_new(struct be_ctx *be_ctx,
         return NULL;
     }
 
-    ret = ad_inherit_opts_if_needed(id_ctx->ipa_options->id->basic,
-                                    ad_options->id->basic, be_ctx->cdb,
-                                    subdom_conf_path, SDAP_SASL_MECH);
+    ret = subdom_inherit_opts_if_needed(id_ctx->ipa_options->id->basic,
+                                        ad_options->id->basic, be_ctx->cdb,
+                                        subdom_conf_path, SDAP_SASL_MECH);
     talloc_free(subdom_conf_path);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE,
@@ -222,6 +357,65 @@ static struct ad_options *ipa_ad_options_new(struct be_ctx *be_ctx,
     return ad_options;
 }
 
+static struct ipa_options *ipa_options_new(struct be_ctx *be_ctx,
+                                           struct ipa_id_ctx *id_ctx,
+                                           struct sss_domain_info *subdom)
+{
+    struct ipa_options *ipa_options = NULL;
+    uint32_t direction;
+    const char *forest;
+    const char *forest_realm;
+    char *subdom_conf_path;
+    int ret;
+
+    /* Trusts are only established with forest roots */
+    direction = subdom->forest_root->trust_direction;
+    forest_realm = subdom->forest_root->realm;
+    forest = subdom->forest_root->forest;
+
+    subdom_conf_path = subdomain_create_conf_path(id_ctx, subdom);
+    if (subdom_conf_path == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "subdom_conf_path failed\n");
+        return NULL;
+    }
+
+    /* In both inbound and outbound trust cases we should be
+     * using trusted domain object in a trusted domain space,
+     * thus we always should be initializing principals/keytabs
+     * as if we are running one-way trust */
+    if (direction & LSA_TRUST_DIRECTION_MASK) {
+        ipa_options = ipa_create_ipa_trust_ctx(id_ctx, be_ctx,
+                                               subdom_conf_path, forest,
+                                               forest_realm, subdom);
+    } else {
+        DEBUG(SSSDBG_CRIT_FAILURE, "Unsupported trust direction!\n");
+        ipa_options = NULL;
+    }
+
+    if (ipa_options == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "Cannot initialize IPA options\n");
+        talloc_free(subdom_conf_path);
+        return NULL;
+    }
+
+    ret = subdom_inherit_opts_if_needed(id_ctx->ipa_options->id->basic,
+                                        ipa_options->id->basic, be_ctx->cdb,
+                                        subdom_conf_path, SDAP_SASL_MECH);
+    talloc_free(subdom_conf_path);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "Failed to inherit option [%s] to sub-domain [%s]. "
+              "This error is ignored but might cause issues or unexpected "
+              "behavior later on.\n",
+              id_ctx->ipa_options->id->basic[SDAP_SASL_MECH].opt_name,
+              subdom->name);
+
+        return NULL;
+    }
+
+    return ipa_options;
+}
+
 
 static errno_t
 ipa_ad_ctx_new(struct be_ctx *be_ctx,
@@ -319,7 +513,7 @@ ipa_ad_ctx_new(struct be_ctx *be_ctx,
     ret = ad_failover_init(ad_options, be_ctx, ad_servers, ad_backup_servers,
                            subdom->realm,
                            service_name, gc_service_name,
-                           subdom->name, use_kdcinfo,
+                           subdom->name, use_kdcinfo, false,
                            n_lookahead_primary, n_lookahead_backup,
                            &ad_options->service);
     if (ret != EOK) {
@@ -342,6 +536,7 @@ ipa_ad_ctx_new(struct be_ctx *be_ctx,
     srv_ctx = ad_srv_plugin_ctx_init(be_ctx, be_ctx, be_ctx->be_res,
                                      default_host_dbs,
                                      ad_id_ctx->ad_options->id,
+                                     ad_id_ctx->ad_options,
                                      id_ctx->server_mode->hostname,
                                      ad_domain,
                                      ad_site_override);
@@ -402,6 +597,163 @@ ipa_ad_ctx_new(struct be_ctx *be_ctx,
     return EOK;
 }
 
+static errno_t
+ipa_ctx_new(struct be_ctx *be_ctx,
+            struct ipa_id_ctx *id_ctx,
+            struct sss_domain_info *subdom,
+            struct ipa_id_ctx **_ipa_id_ctx)
+{
+    struct ipa_options *ipa_options = NULL;
+    struct ipa_id_ctx *ipa_subdom_id_ctx;
+    struct ipa_srv_plugin_ctx *srv_ctx;
+    const char *ipa_domain;
+    char *subdom_conf_path = NULL;
+    char *basedn;
+    struct sdap_domain *sdom;
+    errno_t ret;
+    const char *extra_attrs;
+
+    ipa_domain = subdom->name;
+    DEBUG(SSSDBG_TRACE_LIBS, "Setting up IPA subdomain %s\n", subdom->name);
+
+    subdom_conf_path = subdomain_create_conf_path(id_ctx, subdom);
+    if (subdom_conf_path == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, "subdom_conf_path failed\n");
+        ret = ENOMEM;
+        goto done;
+    }
+
+    ipa_options = ipa_options_new(be_ctx, id_ctx, subdom);
+    if (ipa_options == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "Cannot initialize AD options\n");
+        ret = ENOMEM;
+        goto done;
+    }
+
+    extra_attrs = dp_opt_get_string(id_ctx->sdap_id_ctx->opts->basic,
+                            SDAP_USER_EXTRA_ATTRS);
+    if (extra_attrs != NULL) {
+        DEBUG(SSSDBG_TRACE_ALL,
+              "Setting extra attrs for subdomain [%s] to [%s].\n", ipa_domain,
+                                                                   extra_attrs);
+
+        ret = dp_opt_set_string(ipa_options->id->basic, SDAP_USER_EXTRA_ATTRS,
+                                extra_attrs);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_OP_FAILURE, "dp_opt_set_string failed.\n");
+            goto done;
+        }
+
+        ret = sdap_extend_map_with_list(ipa_options->id, ipa_options->id,
+                                        SDAP_USER_EXTRA_ATTRS,
+                                        ipa_options->id->user_map,
+                                        SDAP_OPTS_USER,
+                                        &ipa_options->id->user_map,
+                                        &ipa_options->id->user_map_cnt);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_OP_FAILURE, "sdap_extend_map_with_list failed.\n");
+            goto done;
+        }
+    } else {
+        DEBUG(SSSDBG_TRACE_ALL, "No extra attrs set.\n");
+    }
+
+    /* ipa_id_ctx and sdap_id_ctx for the ipa subdomain
+     * are initialized in ipa_options_new() */
+    ipa_subdom_id_ctx = ipa_options->id_ctx;
+    ipa_subdom_id_ctx->sdap_id_ctx->opts = ipa_options->id;
+
+    srv_ctx = ipa_srv_plugin_ctx_init(be_ctx, be_ctx->be_res->resolv,
+                                      id_ctx->server_mode->hostname,
+                                      ipa_domain);
+
+    if (srv_ctx == NULL) {
+        DEBUG(SSSDBG_FATAL_FAILURE, "Out of memory?\n");
+        ret = ENOMEM;
+        goto done;
+    }
+    be_fo_set_srv_lookup_plugin(be_ctx, ipa_srv_plugin_send,
+                                ipa_srv_plugin_recv, srv_ctx, "IPA");
+
+   ret = sdap_domain_subdom_add(ipa_subdom_id_ctx->sdap_id_ctx,
+                                ipa_subdom_id_ctx->sdap_id_ctx->opts->sdom,
+                                subdom->parent);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "Cannot initialize sdap domain\n");
+        goto done;
+    }
+
+    sdom = sdap_domain_get(ipa_subdom_id_ctx->sdap_id_ctx->opts, subdom);
+    if (sdom == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, "Cannot get sdap domain\n");
+        ret = EFAULT;
+        goto done;
+    }
+
+    ret = ipa_set_sdap_options(ipa_options, ipa_options->id);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "Cannot set IPA sdap options\n");
+        goto done;
+    }
+
+    ret = domain_to_basedn(be_ctx,
+                           dp_opt_get_string(ipa_options->basic, IPA_DOMAIN),
+                           &basedn);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "domain_to_basedn failure\n");
+        goto done;
+    }
+
+    ipa_set_search_bases(ipa_options,
+                         be_ctx->cdb,
+                         basedn,
+                         subdom_conf_path,
+                         sdom);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, "Cannot set search bases\n");
+        goto done;
+    }
+
+    sdap_inherit_options(subdom->parent->sd_inherit,
+                         id_ctx->sdap_id_ctx->opts,
+                         ipa_subdom_id_ctx->sdap_id_ctx->opts);
+
+    ret = sdap_id_setup_tasks(be_ctx,
+                              ipa_subdom_id_ctx->sdap_id_ctx,
+                              sdom,
+                              ldap_id_enumeration_send,
+                              ldap_id_enumeration_recv,
+                              ipa_subdom_id_ctx->sdap_id_ctx);
+    if (ret != EOK) {
+        goto done;
+    }
+
+    sdom->pvt = ipa_subdom_id_ctx;
+
+    /* Set up the ID mapping object */
+    ipa_subdom_id_ctx->sdap_id_ctx->opts->idmap_ctx =
+        id_ctx->sdap_id_ctx->opts->idmap_ctx;
+
+    /* Set up the certificate mapping context */
+    ipa_subdom_id_ctx->sdap_id_ctx->opts->sdap_certmap_ctx =
+        id_ctx->sdap_id_ctx->opts->sdap_certmap_ctx;
+
+    *_ipa_id_ctx = ipa_subdom_id_ctx;
+
+    ret = EOK;
+done:
+
+    if (ret != EOK) {
+        if (subdom_conf_path != NULL) {
+            talloc_free(subdom_conf_path);
+        }
+        if (ipa_options != NULL) {
+            talloc_free(ipa_options);
+        }
+    }
+    return ret;
+}
+
 struct ipa_getkeytab_state {
     int child_status;
     struct sss_child_ctx_old *child_ctx;
@@ -513,7 +865,7 @@ static void ipa_getkeytab_exec(const char *ccache,
 
     gkt_env[0] = talloc_asprintf(NULL, "KRB5CCNAME=%s", ccache);
     if (gkt_env[0] == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "Failed to format KRB5CCNAME\n");
+        DEBUG(SSSDBG_FATAL_FAILURE, "Failed to format KRB5CCNAME\n");
         exit(1);
     }
 
@@ -522,7 +874,7 @@ static void ipa_getkeytab_exec(const char *ccache,
     ret = unlink(keytab_path);
     if (ret == -1) {
         ret = errno;
-        DEBUG(SSSDBG_CRIT_FAILURE,
+        DEBUG(SSSDBG_FATAL_FAILURE,
               "Failed to unlink the temporary ccname [%d][%s]\n",
               ret, sss_strerror(ret));
         exit(1);
@@ -533,12 +885,12 @@ static void ipa_getkeytab_exec(const char *ccache,
                  "-r", "-s", server, "-p", principal, "-k", keytab_path, NULL,
                  gkt_env);
 
-    DEBUG(SSSDBG_CRIT_FAILURE,
+    DEBUG(SSSDBG_FATAL_FAILURE,
           "execle returned %d, this shouldn't happen!\n", ret);
 
     /* The child should never end up here */
     ret = errno;
-    DEBUG(SSSDBG_CRIT_FAILURE,
+    DEBUG(SSSDBG_FATAL_FAILURE,
           "execle failed [%d][%s].\n", ret, sss_strerror(ret));
     exit(1);
 }
@@ -748,7 +1100,8 @@ static errno_t ipa_server_trusted_dom_setup_1way(struct tevent_req *req)
 
     state->new_keytab = talloc_asprintf(state, "%sXXXXXX", state->keytab);
     if (state->new_keytab == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "Cannot set up ipa_get_keytab\n");
+        DEBUG(SSSDBG_CRIT_FAILURE,
+              "Cannot set up ipa_get_keytab. talloc_asprintf() failed\n");
         return ENOMEM;
     }
 
@@ -902,8 +1255,8 @@ immediate:
 static errno_t ipa_server_create_trusts_step(struct tevent_req *req)
 {
     struct tevent_req *subreq = NULL;
-    struct ipa_ad_server_ctx *trust_iter;
-    struct ipa_ad_server_ctx *trust_i;
+    struct ipa_subdom_server_ctx *trust_iter;
+    struct ipa_subdom_server_ctx *trust_i;
     struct ipa_server_create_trusts_state *state = NULL;
 
     state = tevent_req_data(req, struct ipa_server_create_trusts_state);
@@ -934,12 +1287,11 @@ static errno_t ipa_server_create_trusts_step(struct tevent_req *req)
         }
     }
 
-    /* Refresh all sdap_dom lists in all ipa_ad_server_ctx contexts */
+    /* Refresh all sdap_dom lists in all ipa_subdom_server_ctx contexts */
     DLIST_FOR_EACH(trust_iter, state->id_ctx->server_mode->trusts) {
         struct sdap_domain *sdom_a;
 
-        sdom_a = sdap_domain_get(trust_iter->ad_id_ctx->sdap_id_ctx->opts,
-                                 trust_iter->dom);
+        sdom_a = sdap_domain_get_by_trust_type(trust_iter, trust_iter->dom);
         if (sdom_a == NULL) {
             continue;
         }
@@ -951,8 +1303,7 @@ static errno_t ipa_server_create_trusts_step(struct tevent_req *req)
                 continue;
             }
 
-            sdom_b = sdap_domain_get(trust_i->ad_id_ctx->sdap_id_ctx->opts,
-                                     sdom_a->dom);
+            sdom_b = sdap_domain_get_by_trust_type(trust_i, sdom_a->dom);
             if (sdom_b == NULL) {
                 continue;
             }
@@ -999,26 +1350,48 @@ static void ipa_server_create_trusts_done(struct tevent_req *subreq)
 
 static errno_t ipa_server_create_trusts_ctx(struct tevent_req *req)
 {
-    struct ipa_ad_server_ctx *trust_ctx;
+    struct ipa_subdom_server_ctx *trust_ctx;
     struct ad_id_ctx *ad_id_ctx;
+    struct ipa_id_ctx *ipa_id_ctx;
     errno_t ret;
+    enum ipa_trust_type trust_type;
     struct ipa_server_create_trusts_state *state = NULL;
 
     state = tevent_req_data(req, struct ipa_server_create_trusts_state);
 
-    ret = ipa_ad_ctx_new(state->be_ctx, state->id_ctx, state->domiter, &ad_id_ctx);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE,
-              "Cannot create ad_id_ctx for subdomain %s\n", state->domiter->name);
-        return ret;
-    }
+    trust_type = state->domiter->forest_root->trust_type;
 
-    trust_ctx = talloc(state->id_ctx->server_mode, struct ipa_ad_server_ctx);
+    trust_ctx = talloc(state->id_ctx->server_mode, struct ipa_subdom_server_ctx);
     if (trust_ctx == NULL) {
         return ENOMEM;
     }
     trust_ctx->dom = state->domiter;
-    trust_ctx->ad_id_ctx = ad_id_ctx;
+    trust_ctx->type = trust_type;
+
+    /* Previously stored AD trusted domains dont contain trust type attr */
+    if (trust_type != IPA_TRUST_AD && trust_type != IPA_TRUST_IPA) {
+        trust_type = IPA_TRUST_AD;
+    }
+
+    if (trust_type == IPA_TRUST_AD) {
+        ret = ipa_ad_ctx_new(state->be_ctx, state->id_ctx, state->domiter, &ad_id_ctx);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_OP_FAILURE,
+                  "Cannot create ad_id_ctx for subdomain %s\n", state->domiter->name);
+            return ret;
+        }
+
+        trust_ctx->id_ctx.ad_id_ctx = ad_id_ctx;
+    } else if (trust_type == IPA_TRUST_IPA) {
+        ret = ipa_ctx_new(state->be_ctx, state->id_ctx, state->domiter, &ipa_id_ctx);
+        if (ret != EOK) {
+            DEBUG(SSSDBG_OP_FAILURE,
+                  "Cannot create ipa_id_ctx for subdomain %s\n", state->domiter->name);
+            return ret;
+        }
+
+        trust_ctx->id_ctx.ipa_id_ctx = ipa_id_ctx;
+    }
 
     DLIST_ADD(state->id_ctx->server_mode->trusts, trust_ctx);
     return EOK;
@@ -1034,7 +1407,7 @@ void ipa_ad_subdom_remove(struct be_ctx *be_ctx,
                           struct ipa_id_ctx *id_ctx,
                           struct sss_domain_info *subdom)
 {
-    struct ipa_ad_server_ctx *iter;
+    struct ipa_subdom_server_ctx *iter;
     struct sdap_domain *sdom;
 
     if (dp_opt_get_bool(id_ctx->ipa_options->basic,
@@ -1052,10 +1425,10 @@ void ipa_ad_subdom_remove(struct be_ctx *be_ctx,
         return;
     }
 
-    sdom = sdap_domain_get(iter->ad_id_ctx->sdap_id_ctx->opts, subdom);
+    sdom = sdap_domain_get_by_trust_type(iter, subdom);
     if (sdom == NULL) return;
 
-    sdap_domain_remove(iter->ad_id_ctx->sdap_id_ctx->opts, subdom);
+    sdap_domain_remove_by_trust_type(iter, subdom);
     DLIST_REMOVE(id_ctx->server_mode->trusts, iter);
 
     /* terminate all requests for this subdomain so we can free it */
@@ -1063,7 +1436,7 @@ void ipa_ad_subdom_remove(struct be_ctx *be_ctx,
     talloc_zfree(sdom);
 }
 
-struct ipa_ad_subdom_reinit_state {
+struct ipa_trusted_subdom_reinit_state {
     struct tevent_context *ev;
     struct be_ctx *be_ctx;
     struct ipa_id_ctx *id_ctx;
@@ -1088,9 +1461,9 @@ static void create_trusts_at_startup(struct tevent_context *ev,
                                      void *pvt)
 {
     struct tevent_req *req;
-    struct ipa_ad_subdom_reinit_state *state;
+    struct ipa_trusted_subdom_reinit_state *state;
 
-    state = talloc_get_type(pvt, struct ipa_ad_subdom_reinit_state);
+    state = talloc_get_type(pvt, struct ipa_trusted_subdom_reinit_state);
 
     req = ipa_server_create_trusts_send(state, state->ev, state->be_ctx,
                                         state->id_ctx, state->parent);
@@ -1104,16 +1477,16 @@ static void create_trusts_at_startup(struct tevent_context *ev,
     return;
 }
 
-static errno_t ipa_ad_subdom_reinit(TALLOC_CTX *mem_ctx,
-                                    struct tevent_context *ev,
-                                    struct be_ctx *be_ctx,
-                                    struct ipa_id_ctx *id_ctx,
-                                    struct sss_domain_info *parent)
+static errno_t ipa_trusted_subdom_reinit(TALLOC_CTX *mem_ctx,
+                                         struct tevent_context *ev,
+                                         struct be_ctx *be_ctx,
+                                         struct ipa_id_ctx *id_ctx,
+                                         struct sss_domain_info *parent)
 {
     struct tevent_immediate *imm;
-    struct ipa_ad_subdom_reinit_state *state;
+    struct ipa_trusted_subdom_reinit_state *state;
 
-    state = talloc(mem_ctx, struct ipa_ad_subdom_reinit_state);
+    state = talloc(mem_ctx, struct ipa_trusted_subdom_reinit_state);
     if (state == NULL) {
         return ENOMEM;
     }
@@ -1138,8 +1511,8 @@ static errno_t ipa_ad_subdom_reinit(TALLOC_CTX *mem_ctx,
     return EOK;
 }
 
-int ipa_ad_subdom_init(struct be_ctx *be_ctx,
-                       struct ipa_id_ctx *id_ctx)
+int ipa_trusted_subdom_init(struct be_ctx *be_ctx,
+                            struct ipa_id_ctx *id_ctx)
 {
     char *realm;
     char *hostname;
@@ -1202,10 +1575,10 @@ int ipa_ad_subdom_init(struct be_ctx *be_ctx,
         }
     }
 
-    ret = ipa_ad_subdom_reinit(be_ctx, be_ctx->ev,
+    ret = ipa_trusted_subdom_reinit(be_ctx, be_ctx->ev,
                                be_ctx, id_ctx, be_ctx->domain);
     if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, "ipa_ad_subdom_refresh failed.\n");
+        DEBUG(SSSDBG_OP_FAILURE, "ipa_trusted_subdom_reinit failed.\n");
         return ret;
     }
 
