@@ -529,7 +529,14 @@ errno_t sss_krb5_cc_verify_ccache(const char *ccname, uid_t uid, gid_t gid,
 
     mcred.client = princ;
     mcred.server = tgt_princ;
-    mcred.times.endtime = time(NULL);
+    /* Type krb5_timestamp is a signed 32-bit integer, so we need to convert the
+     * 64-bit time_t value returned by time(). Just keeping the lower 32 bits
+     * should be enough as Kerberos seems to be planing on making this time
+     * unsigned to avoid the Y2K38 problem.
+     * Please check:
+     * https://web.mit.edu/kerberos/krb5-latest/doc/appdev/y2038.html
+     */
+    mcred.times.endtime = time(NULL) & 0xFFFFFFFF;
 
     kerr = krb5_cc_retrieve_cred(cc->context, cc->ccache,
                                  KRB5_TC_MATCH_TIMES, &mcred, &cred);
@@ -630,12 +637,12 @@ errno_t get_ccache_file_data(const char *ccache_file, const char *client_name,
     krb5_free_cred_contents(ctx, &cred);
 
     kerr = krb5_cc_close(ctx, cc);
+    cc = NULL;
     if (kerr != 0) {
         KRB5_DEBUG(SSSDBG_OP_FAILURE, ctx, kerr);
         DEBUG(SSSDBG_CRIT_FAILURE, "krb5_cc_close failed.\n");
         goto done;
     }
-    cc = NULL;
 
     kerr = 0;
 
@@ -774,14 +781,14 @@ done:
         talloc_free(mem_name);
     }
 
-    free(ccache_name);
+    krb5_free_string(kctx, ccache_name);
     krb5_free_principal(kctx, princ);
 
     if (krb5_cc_close(kctx, ccache) != 0) {
         DEBUG(SSSDBG_OP_FAILURE, "krb5_cc_close failed.\n");
     }
 
-    if (krb5_cc_close(kctx, mem_ccache) != 0) {
+    if ((mem_ccache != NULL) && (krb5_cc_close(kctx, mem_ccache) != 0)) {
         DEBUG(SSSDBG_OP_FAILURE, "krb5_cc_close failed.\n");
     }
 
