@@ -23,6 +23,7 @@
 
 #include "db/sysdb_ssh.h"
 #include "util/util.h"
+#include "util/sss_chain_id.h"
 #include "providers/data_provider.h"
 #include "responder/common/cache_req/cache_req_plugin.h"
 
@@ -86,7 +87,7 @@ cache_req_host_by_name_dp_send(TALLOC_CTX *mem_ctx,
     return sbus_call_dp_dp_hostHandler_send(mem_ctx, be_conn->conn,
                                             be_conn->bus_name, SSS_BUS_PATH,
                                             0, data->name.name, data->alias,
-                                            cr->rctx->client_id_num);
+                                            sss_chain_id_get());
 }
 
 static bool
