@@ -30,15 +30,7 @@
 static inline bool
 attr_in_list(const char **list, size_t nlist, const char *str)
 {
-    size_t i;
-
-    for (i = 0; i < nlist; i++) {
-        if (strcasecmp(list[i], str) == 0) {
-            break;
-        }
-    }
-
-    return (i < nlist) ? true : false;
+    return string_in_list_size(str, list, nlist, false);
 }
 
 const char **parse_attr_list_ex(TALLOC_CTX *mem_ctx, const char *conf_str,
@@ -118,6 +110,11 @@ const char **parse_attr_list_ex(TALLOC_CTX *mem_ctx, const char *conf_str,
             continue;
         }
 
+        /* If the attribute is already in the list, skip it */
+        if (attr_in_list(list, li, allow[i])) {
+            continue;
+        }
+
         list[li] = talloc_strdup(list, allow[i]);
         if (list[li] == NULL) {
             goto done;
@@ -136,6 +133,11 @@ const char **parse_attr_list_ex(TALLOC_CTX *mem_ctx, const char *conf_str,
                 continue;
             }
 
+            /* If the attribute is already in the list, skip it */
+            if (attr_in_list(list, li, defaults[i])) {
+                continue;
+            }
+
             list[li] = talloc_strdup(list, defaults[i]);
             if (list[li] == NULL) {
                 goto done;
@@ -153,49 +155,6 @@ done:
     return res;
 }
 
-char *sss_resp_create_fqname(TALLOC_CTX *mem_ctx,
-                             struct resp_ctx *rctx,
-                             struct sss_domain_info *dom,
-                             bool name_is_upn,
-                             const char *orig_name)
-{
-    TALLOC_CTX *tmp_ctx;
-    char *name;
-
-    tmp_ctx = talloc_new(NULL);
-    if (tmp_ctx == NULL) {
-        return NULL;
-    }
-
-    name = sss_get_cased_name(tmp_ctx, orig_name, dom->case_sensitive);
-    if (name == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "sss_get_cased_name failed\n");
-        talloc_free(tmp_ctx);
-        return NULL;
-    }
-
-    name = sss_reverse_replace_space(tmp_ctx, name, rctx->override_space);
-    if (name == NULL) {
-        DEBUG(SSSDBG_CRIT_FAILURE, "sss_reverse_replace_space failed\n");
-        talloc_free(tmp_ctx);
-        return NULL;
-    }
-
-
-    if (name_is_upn == false) {
-        name = sss_create_internal_fqname(tmp_ctx, name, dom->name);
-        if (name == NULL) {
-            DEBUG(SSSDBG_CRIT_FAILURE, "sss_create_internal_fqname failed\n");
-            talloc_free(tmp_ctx);
-            return NULL;
-        }
-    }
-
-    name = talloc_steal(mem_ctx, name);
-    talloc_free(tmp_ctx);
-    return name;
-}
-
 struct resp_resolve_group_names_state {
     struct tevent_context *ev;
     struct resp_ctx *rctx;
