@@ -24,6 +24,11 @@
 #include "util/authtok-utils.h"
 #include "sss_client/sss_cli.h"
 
+#define IS_SC_AUTHTOK(tok) ( \
+    sss_authtok_get_type((tok)) == SSS_AUTHTOK_TYPE_SC_PIN \
+        || sss_authtok_get_type((tok)) == SSS_AUTHTOK_TYPE_SC_KEYPAD)
+
+
 /* Use sss_authtok_* accessor functions instead of struct sss_auth_token
  */
 struct sss_auth_token;
@@ -391,7 +396,7 @@ errno_t sss_authtok_get_2fa_single(struct sss_auth_token *tok,
  *
  * @param tok        A pointer to an sss_auth_token structure to change, also
  *                   used as a memory context to allocate the internal data.
- * @param password   A string where the two authentication factors are
+ * @param str        A string where the two authentication factors are
  *                   concatenated together
  * @param len        The length of the string or, if 0 is passed,
  *                   then strlen(password) will be used internally.
@@ -402,4 +407,133 @@ errno_t sss_authtok_get_2fa_single(struct sss_auth_token *tok,
 errno_t sss_authtok_set_2fa_single(struct sss_auth_token *tok,
                                    const char *str, size_t len);
 
+/**
+ * @brief Returns a const string if the auth token is of type
+          SSS_AUTHTOK_TYPE_OAUTH2, otherwise it returns an error
+ *
+ * @param tok    A pointer to an sss_auth_token
+ * @param pwd    A pointer to a const char *, that will point to a null
+ *               terminated string
+ * @param len    The length of the credential string
+ *
+ * @return       EOK on success
+ *               ENOENT if the token is empty
+ *               EACCESS if the token is not a password token
+ */
+errno_t sss_authtok_get_oauth2(struct sss_auth_token *tok,
+                               const char **str, size_t *len);
+
+/**
+ * @brief Set one-time password into an auth token, replacing any previous data.
+ *
+ * @param tok        A pointer to an sss_auth_token structure to change, also
+ *                   used as a memory context to allocate the internal data.
+ * @param str   A string that holds the one-time password.
+ * @param len        The length of the string or, if 0 is passed,
+ *                   then strlen(password) will be used internally.
+ *
+ * @return       EOK on success
+ *               ENOMEM on error
+ */
+errno_t sss_authtok_set_oauth2(struct sss_auth_token *tok,
+                               const char *str, size_t len);
+/**
+ * @brief Returns a const string if the auth token is of type
+          SSS_AUTHTOK_TYPE_PASSKEY_REPLY, otherwise it returns an error
+ *
+ * @param tok    A pointer to an sss_auth_token
+ * @param str    A string that holds the passkey assertion data
+ * @param len    The length of the credential string
+ *
+ * @return       EOK on success
+ *               ENOENT if the token is empty
+ *               EACCESS if the token is not a passkey token
+ */
+errno_t sss_authtok_set_passkey_reply(struct sss_auth_token *tok,
+                                      const char *str, size_t len);
+/**
+ * @brief Returns a const string if the auth token is of type
+          SSS_AUTHTOK_TYPE_PASSKEY_REPLY, otherwise it returns an error
+ *
+ * @param tok    A pointer to an sss_auth_token
+ * @param str    A pointer to a const char *, that will point to a null
+ *               terminated string
+ * @param len    The length of the credential string
+ *
+ * @return       EOK on success
+ *               ENOENT if the token is empty
+ *               EACCESS if the token is not a password token
+ */
+errno_t sss_authtok_get_passkey_reply(struct sss_auth_token *tok,
+                                      const char **str, size_t *len);
+
+/**
+ * @brief Returns a const string if the auth token is of type
+          SSS_AUTHTOK_TYPE_PASSKEY, otherwise it returns an error
+ *
+ * @param mem_ctx    Parent talloc context to attach to
+ * @param tok    A pointer to an sss_auth_token
+ * @param prompt A pointer to a const char *, that will point to a null
+ *               terminated string
+ * @param key    A pointer to a const char *, that will point to a null
+ *               terminated string
+ * @param pin    A pointer to a const char *, that will point to a null
+ *               terminated string
+ *
+ * @return       EOK on success
+ *               ENOENT if the token is empty
+ *               EACCESS if the token is not a password token
+ */
+errno_t sss_authtok_get_passkey(TALLOC_CTX *mem_ctx,
+                                struct sss_auth_token *tok,
+                                const char **_prompt, const char **_key,
+                                const char **_pin, size_t *_pin_len);
+/**
+ * @brief Returns a const string if the auth token is of type
+          SSS_AUTHTOK_TYPE_PASSKEY, otherwise it returns the error code
+ *
+ * @param tok    A pointer to an sss_auth_token
+ * @param pwd    A pointer to a const char *, that will point to a null
+ *               terminated string
+ * @param len    The length of the credential string
+ *
+ * @return       EOK on success
+ *               ENOENT if the token is empty
+ *               EACCESS if the token is not a password token
+ */
+errno_t sss_authtok_get_passkey_pin(struct sss_auth_token *tok,
+                                    const char **pin, size_t *len);
+
+/**
+ * @brief Set passkey kerberos preauth credentials into an auth token,
+ *        replacing any previous data.
+ *
+ * @param tok        A pointer to an sss_auth_token structure to change, also
+ *                   used as a memory context to allocate the internal data.
+ * @param pin        A string that holds the passkey PIN.
+ * @param len        The length of the string or, if 0 is passed,
+ *                   then strlen(password) will be used internally.
+ *
+ * @return       EOK on success
+ *               ENOMEM on error
+ */
+errno_t sss_authtok_set_passkey_krb(struct sss_auth_token *tok,
+                                    const char *prompt, const char *key,
+                                    const char *pin);
+
+/**
+ * @brief Set PAM stacked credentials in a single string into an auth token,
+ *        replacing any previous data
+ *
+ * @param tok        A pointer to an sss_auth_token structure to change, also
+ *                   used as a memory context to allocate the internal data.
+ * @param str        A string where the stacked credential is found.
+ * @param len        The length of the string or, if 0 is passed,
+ *                   then strlen(password) will be used internally.
+ *
+ * @return       EOK on success
+ *               ENOMEM on error
+ */
+errno_t sss_authtok_set_pam_stacked(struct sss_auth_token *tok,
+                                    const char *str, size_t len);
 #endif /*  __AUTHTOK_H__ */
