@@ -88,6 +88,23 @@ char *expand_homedir_template(TALLOC_CTX *mem_ctx,
                 talloc_free(username);
                 break;
 
+            case 'l':
+                if (homedir_ctx->username == NULL) {
+                    DEBUG(SSSDBG_CRIT_FAILURE,
+                          "Cannot expand first letter of user name template "
+                          "because user name is empty.\n");
+                    goto done;
+                }
+                username = sss_output_name(tmp_ctx, homedir_ctx->username,
+                                           case_sensitive, 0);
+                if (username == NULL) {
+                    goto done;
+                }
+
+                result = talloc_asprintf_append(result, "%s%c", p, username[0]);
+                talloc_free(username);
+                break;
+
             case 'U':
                 if (homedir_ctx->uid == 0) {
                     DEBUG(SSSDBG_CRIT_FAILURE, "Cannot expand uid template "
@@ -129,13 +146,25 @@ char *expand_homedir_template(TALLOC_CTX *mem_ctx,
                 break;
 
             case 'o':
+            case 'h':
                 if (homedir_ctx->original == NULL) {
                     DEBUG(SSSDBG_CRIT_FAILURE,
                           "Original home directory for %s is not available, "
                            "using empty string\n", homedir_ctx->username);
                     orig = "";
                 } else {
-                    orig = homedir_ctx->original;
+                    if (*n == 'o') {
+                        orig = homedir_ctx->original;
+                    } else {
+                        orig = sss_tc_utf8_str_tolower(tmp_ctx,
+                                                       homedir_ctx->original);
+                        if (orig == NULL) {
+                            DEBUG(SSSDBG_CRIT_FAILURE,
+                                  "Failed to lowercase the original home "
+                                  "directory.\n");
+                            goto done;
+                        }
+                    }
                 }
                 result = talloc_asprintf_append(result, "%s%s", p, orig);
                 break;
