@@ -22,11 +22,11 @@
 #ifndef _STRTONUM_H_
 #define _STRTONUM_H_
 
-#include <ctype.h>
-#include <stdlib.h>
 #include <stdint.h>
 
 int32_t strtoint32(const char *nptr, char **endptr, int base);
 uint32_t strtouint32(const char *nptr, char **endptr, int base);
 
+uint16_t strtouint16(const char *nptr, char **endptr, int base);
+
 #endif /* _STRTONUM_H_ */
